@@ -1,11 +1,18 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2015 openpyxl
+# Copyright (c) 2010-2017 openpyxl
 
 from openpyxl.compat import safe_string
-from openpyxl.descriptors import Set, Typed, Bool, Alias
+from openpyxl.descriptors import (
+    NoneSet,
+    Typed,
+    Bool,
+    Alias,
+    Sequence,
+    Integer,
+)
+from openpyxl.descriptors.serialisable import Serialisable
 
 from .colors import ColorDescriptor
-from .hashable import HashableObject
 
 
 BORDER_NONE = None
@@ -24,9 +31,7 @@ BORDER_THICK = 'thick'
 BORDER_THIN = 'thin'
 
 
-class Side(HashableObject):
-
-    spec = """Actually to BorderPr 18.8.6"""
+class Side(Serialisable):
 
     """Border options for use in styles.
     Caution: if you do not specify a border_style, other attributes will
@@ -36,10 +41,10 @@ class Side(HashableObject):
                   'color')
 
     color = ColorDescriptor(allow_none=True)
-    style = Set(values=(BORDER_NONE, BORDER_DASHDOT, BORDER_DASHDOTDOT,
-                        BORDER_DASHED, BORDER_DOTTED, BORDER_DOUBLE, BORDER_HAIR, BORDER_MEDIUM,
-                        BORDER_MEDIUMDASHDOT, BORDER_MEDIUMDASHDOTDOT, BORDER_MEDIUMDASHED,
-                        BORDER_SLANTDASHDOT, BORDER_THICK, BORDER_THIN))
+    style = NoneSet(values=('dashDot','dashDotDot', 'dashed','dotted',
+                            'double','hair', 'medium', 'mediumDashDot', 'mediumDashDotDot',
+                            'mediumDashed', 'slantDashDot', 'thick', 'thin')
+                    )
     border_style = Alias('style')
 
     def __init__(self, style=None, color=None, border_style=None):
@@ -48,16 +53,11 @@ class Side(HashableObject):
         self.style = style
         self.color = color
 
-    def __iter__(self):
-        for key in ("style",):
-            value = getattr(self, key)
-            if value:
-                yield key, safe_string(value)
 
-
-class Border(HashableObject):
+class Border(Serialisable):
     """Border positioning for use in styles."""
 
+    tagname = "border"
 
     __fields__ = ('left',
                   'right',
@@ -67,11 +67,15 @@ class Border(HashableObject):
                   'diagonal_direction',
                   'vertical',
                   'horizontal')
+    __elements__ = ('start', 'end', 'left', 'right', 'top', 'bottom',
+                    'diagonal', 'vertical', 'horizontal')
 
     # child elements
-    left = Typed(expected_type=Side)
-    right = Typed(expected_type=Side)
-    top = Typed(expected_type=Side)
+    start = Typed(expected_type=Side, allow_none=True)
+    end = Typed(expected_type=Side, allow_none=True)
+    left = Typed(expected_type=Side, allow_none=True)
+    right = Typed(expected_type=Side, allow_none=True)
+    top = Typed(expected_type=Side, allow_none=True)
     bottom = Typed(expected_type=Side)
     diagonal = Typed(expected_type=Side, allow_none=True)
     vertical = Typed(expected_type=Side, allow_none=True)
@@ -84,7 +88,7 @@ class Border(HashableObject):
     def __init__(self, left=Side(), right=Side(), top=Side(),
                  bottom=Side(), diagonal=Side(), diagonal_direction=None,
                  vertical=None, horizontal=None, diagonalUp=False, diagonalDown=False,
-                 outline=True):
+                 outline=True, start=None, end=None):
         self.left = left
         self.right = right
         self.top = top
@@ -96,21 +100,15 @@ class Border(HashableObject):
         self.diagonalUp = diagonalUp
         self.diagonalDown = diagonalDown
         self.outline = outline
-
-    @property
-    def children(self):
-        for key in ('left', 'right', 'top', 'bottom', 'diagonal', 'vertical',
-                    'horizontal'):
-            value = getattr(self, key)
-            if value is not None:
-                yield key, value
+        self.start = start
+        self.end = end
 
     def __iter__(self):
-        """
-        Unset outline defaults to True, others default to False
-        """
-        for key in ('diagonalUp', 'diagonalDown', 'outline'):
-            value = getattr(self, key)
-            if (key == "outline" and not value
-                or key != "outline" and value):
-                yield key, safe_string(value)
+        for attr in self.__attrs__:
+            value = getattr(self, attr)
+            if value and attr != "outline":
+                yield attr, safe_string(value)
+            elif attr == "outline" and not value:
+                yield attr, safe_string(value)
+
+DEFAULT_BORDER = Border()
